from .battery_advisor import BatteryAdvisor, VERSION
import argparse
import logging
import os
from .utils import _get_project_root


def cli():
    log_path = os.path.join(_get_project_root(), "advisor.log")

    logging.basicConfig(
        level=logging.DEBUG,
        format="[%(asctime)s ] (%(module)s.%(funcName)s | %(levelname)s) %(message)s",
        datefmt="%d/%m/%Y @ %H:%M:%S %Z",
        filename=log_path,
    )

    parser = argparse.ArgumentParser(
        prog="Battery Advisor",
        description="A simple tool to monitor and notify about battery status. Built with Python.",
        epilog="Made with ❤️ by Jorge Hernández.",
    )
    parser.add_argument("--version", action="version", version=f"%(prog)s {VERSION}")
    parser.add_argument("--clean", action="store_true", help="Use default settings.")

    args = parser.parse_args()
    BatteryAdvisor(clean=args.clean).start()



import os
import subprocess

import psutil

EXPIRE_TIME = 600000  # 10 minutes


def _get_project_root() -> str:
    usr_home = os.path.expanduser("~")
    root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    absolute_root = root.replace(usr_home, "$HOME")
    return os.path.expandvars(absolute_root)


def _get_path_icon():
    icon_path = _get_project_root() + "/icon.png"
    return os.path.expandvars(icon_path)


def get_battery_status() -> tuple[int, bool]:
    """Returns the battery percentage and if it is plugged in"""
    batt = psutil.sensors_battery()
    return batt.percent, batt.power_plugged


def execute_action(action_name: str, actions: dict[str, list[str]]) -> None:
    """Executes specified action

    Parameters
    ----------
    action : list[str]
        The name of the action to execute
    """

    try:
        subprocess.run(actions[action_name], stdout=subprocess.PIPE)
    except Exception as e:
        print("Failed to perform action: ", e)



import pystray
from PIL import Image
from PIL.ImageFile import ImageFile

from .utils import _get_project_root


def _get_icon_from_image(image_path: str) -> ImageFile:
    """Returns an image object from a file path"""
    return Image.open(image_path)


def get_icon(menu: pystray.Menu):
    icon_path = _get_project_root() + "/icon.png"
    i = pystray.Icon(
        name="Battery Advisor",
        icon=_get_icon_from_image(icon_path),
        title="Battery Advisor",
        menu=menu,
    )

    if not i.HAS_MENU:
        # Warn no menu
        print("No menu available for this platform.")

    return i



import os
import toml
from typing import Type
from .types import SettingsFile
from .utils import _get_project_root

user_settings_path = os.path.expanduser("~/.config/battery-advisor/settings.toml")
default_settings_path = _get_project_root() + "/defaultSettings.toml"


def load_settings(load_default: bool = False) -> SettingsFile:
    """Loads the settings from the settings.toml file and returns a dictionary with the settings

    Parameters
    ----------
    load_default : bool, optional
        If True, the default settings will be loaded instead of the user settings.
    """

    if load_default:
        print("Loading default settings...")
        with open(default_settings_path) as f:
            return toml.load(f)

    path = (
        user_settings_path
        if os.path.exists(user_settings_path)
        else default_settings_path
    )

    if not os.path.exists(user_settings_path):
        print("User settings not found. Using default settings.")

    with open(path) as f:
        return toml.load(f)


class Settings:
    def __init__(self, settings: SettingsFile):
        """Creates a settings object based on the settings.toml file.
        NOTE: This class is not meant to be instantiated directly. Use the load() method instead.
        """

        self.check_interval: int = int(settings["advisor"]["check_interval"])
        self.remind_time: int = int(settings["advisor"]["remind_time"])

        # Tresholds
        self.low_battery_treshold = settings["tresholds"]["low_battery_treshold"]
        self.critical_battery_treshold = settings["tresholds"][
            "critical_battery_treshold"
        ]
        self.battery_action_treshold = settings["tresholds"]["battery_action_treshold"]

        # Alert Options
        self.low_battery_options = settings["advisor"]["low_battery_options"]
        self.critical_battery_options = settings["advisor"]["critical_battery_options"]
        self.battery_action = settings["advisor"]["battery_action"]

        self.actions: dict[str, list[str]] = settings["actions"]

        # Notification Settings
        self.notify_plugged = settings["advisor"]["notify_plugged"]
        self.notify_unplugged = settings["advisor"]["notify_unplugged"]

    @classmethod
    def load(cls: Type["Settings"], clean: bool = False) -> "Settings":
        """Loads the settings from the settings.toml file and returns a Settings object

        Parameters
        ----------
        clean : bool, optional
            If True, the default settings will be loaded instead of the user settings.
        """

        return cls(load_settings(load_default=clean))



from typing import Literal, TypedDict
from enum import Enum


class Tresholds(TypedDict):
    low_battery_treshold: int
    critical_battery_treshold: int
    battery_action_treshold: int


class Advisor(TypedDict):
    notify_plugged: bool
    notify_unplugged: bool
    battery_action: str
    low_battery_options: list[str]
    critical_battery_options: list[str]
    remind_time: int
    check_interval: int


class SettingsFile(TypedDict):
    tresholds: Tresholds
    advisor: Advisor
    actions: dict[str, list[str]]


class BatteryReport(Enum):
    LOW = 0
    CRITICAL = 1
    ACTION = 2



import gi

gi.require_version("Gtk", "3.0")
gi.require_version("Pango", "1.0")
from gi.repository import Gtk, Pango

from ..utils import execute_action


class MessageAlert(Gtk.Dialog):
    def __init__(
        self,
        title: str,
        message: str,
    ):
        super().__init__(title, transient_for=None)

        content = self.get_content_area()
        _title_label = Gtk.Label(margin_bottom=13, margin_top=10)
        _title_label.set_markup(f"<b>{title}</b>")
        _msg_label = Gtk.Label(
            message, margin_bottom=12, margin_start=10, margin_end=10
        )

        # Increase text size
        _title_label.override_font(Pango.FontDescription("Ubuntu 12"))
        _msg_label.override_font(Pango.FontDescription("Ubuntu 10"))

        content.add(_title_label)
        content.add(_msg_label)

        if self.__class__ == MessageAlert:
            self.add_button("Close", Gtk.ResponseType.CLOSE)

        self.show_all()

    def run(self):
        response = super().run()
        if response == -4 or response == -7 or response == -1:
            response = 0

        return response


class AlertWithButtons(MessageAlert):
    def __init__(self, title: str, message, actions: list[str]):
        super().__init__(message=message, title=title)

        for i, action in enumerate(actions):
            self.add_button(action.capitalize(), i)

        self.show_all()



import threading
import time

from pystray import Menu, MenuItem
from .settings_loader import Settings
from .tray import get_icon
from .utils import execute_action, get_battery_status
from .notifications import notify, alert_with_options, alert
from .types import BatteryReport
from typing import Optional
from datetime import datetime
import logging
from systemd.journal import JournalHandler

VERSION = "1.1.0"
logger = logging.getLogger(__name__)
logger.addHandler(JournalHandler())


class BatteryAdvisor:
    """Base Program that manages SysTray Icon and the battery checker service."""

    def __init__(self, clean: bool = False):
        """
        Initializes the Battery Advisor program.

        Parameters
        ----------
        clean : bool, optional
            If True, the program will use default settings.
        """

        logging.debug("==== Initializing Battery Advisor... ====")
        self.running = True
        self.settings = Settings.load(clean)

    def get_battery_reports(self) -> Optional[BatteryReport]:
        """Returns thet status that needs to be reported to the user"""

        batt_percent, plugged = get_battery_status()
        logger.info(f"Battery Status: {batt_percent}% | Plugged: {plugged}")

        if plugged:
            return None

        if batt_percent <= self.settings.battery_action_treshold:
            return BatteryReport.ACTION

        if batt_percent <= self.settings.critical_battery_treshold:
            return BatteryReport.CRITICAL

        if batt_percent <= self.settings.low_battery_treshold:
            return BatteryReport.LOW

        return None

    def _battery_checker(self):
        """
        Service that checks the battery status and notifies the user.
        This one runs in a separate thread because the SysTray icon must run on the main thread.
        """

        logging.info("Starting battery checker...")

        # Always get initial status to avoid false notifications
        _, was_plugged = get_battery_status()
        remind_timestamp = datetime.now()

        while True:
            if not self.running:
                time.sleep(3)
                continue

            _, plugged = get_battery_status()

            # Battery Plugged in notifications
            if plugged != was_plugged:
                was_plugged = plugged
                if plugged and self.settings.notify_plugged:
                    notify("Battery Plugged In", "Battery is now charging")
                elif not plugged and self.settings.notify_unplugged:
                    notify("Battery Unplugged", "Battery is now discharging.")

            report = self.get_battery_reports()
            logging.info("Battery Report:", report)

            if report is None:
                time.sleep(self.settings.check_interval)
                continue

            if report == BatteryReport.ACTION:
                battery_action = self.settings.battery_action
                alert(message=f"Your battery will {battery_action.capitalize()} soon.")
                time.sleep(3)
                execute_action(battery_action, self.settings.actions)

            if report == BatteryReport.CRITICAL:
                alert_with_options(
                    message="Your battery is critically low. Please plug in your charger.",
                    options=self.settings.critical_battery_options,
                    title="Battery Critically Low",
                )

            # Don't sleep for the amount of time to remind the user again.
            # Instead, check if the time has passed using timestamps.

            # This is to avoid excluding critical notifications or if a long remind time is set
            # or plugged/unplugged notifications because of sleep.
            # More like a QoL feature. ;)
            if (
                report == BatteryReport.LOW
                and remind_timestamp.timestamp() <= datetime.now().timestamp()
            ):
                r = alert_with_options(
                    "Battery is low. Please plug in your charger.",
                    self.settings.low_battery_options,
                    title="Battery Low",
                )

                selected_action = self.settings.low_battery_options[r]

                if selected_action == "remind":
                    remind_timestamp = datetime.fromtimestamp(
                        remind_timestamp.timestamp() + self.settings.remind_time
                    )

                else:
                    execute_action(selected_action, self.settings.actions)

            time.sleep(self.settings.check_interval)

    def _on_enabled_click(self, icon, item):
        self.running = not self.running
        logging.warning(
            "Battery Advisor is now", "enabled" if self.running else "disabled"
        )

    def start(self):
        batt_thread = threading.Thread(target=self._battery_checker, daemon=True)
        batt_thread.start()

        menu = Menu(
            MenuItem(
                text="Enabled",
                checked=lambda item: self.running,
                action=self._on_enabled_click,
            ),
            MenuItem(text=f"Version: {VERSION}", checked=None, action=None),
        )
        get_icon(menu).run()
        logging.info("Exiting...")
        return 0



import subprocess

from .utils import _get_path_icon, execute_action
from .gui.alerts import AlertWithButtons, MessageAlert

EXPIRE_TIME = 160000


def notify(title: str, message: str):
    """Sends a notification to the user"""
    subprocess.run(["notify-send", title, message, f"--icon={_get_path_icon()}"])


def alert(message: str, title: str = "Battery Advisor"):
    """Sends a popup to the user"""
    dialog = MessageAlert(message=message, title=title)
    dialog.run()
    dialog.destroy()


def alert_with_options(
    message: str, options: list[str], title: str = "Battery Advisor"
) -> int:
    """Sends a popup with a close option to the user.

    Returns
    -------
    int
        The selected option index
    """

    dialog = AlertWithButtons(title=title, message=message, actions=options)
    selection = dialog.run()
    dialog.destroy()
    return selection
